# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""
Server Widget

Includes a single ExplorerWidget with a stacked form widgets

For a single server connection only.

"""

import os,sys,traceback
from PyQt4 import QtCore,QtGui

try:
	_fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
	_fromUtf8 = lambda s: s

import explorerwidget
from rprj.formulator import FAssociation

class ServerWidget(QtGui.QWidget):
	def __init__(self,parent,objectName):
		QtGui.QWidget.__init__(self,parent)
		# UI
		self.setObjectName(_fromUtf8("serverwidget"))
		self.layout = QtGui.QVBoxLayout(self)
		self.layout.setMargin(0)
		self.layout.setObjectName(_fromUtf8("layout"))
		self.splitter = QtGui.QSplitter(self)
		self.splitter.setOrientation(QtCore.Qt.Horizontal)
		self.splitter.setObjectName(_fromUtf8("splitter"))
		self.layout.addWidget(self.splitter)
		self.explorer = explorerwidget.ExplorerWidget(self.splitter,"TestWidget")
		self.explorer.container = self
		self.stackedWidget = QtGui.QStackedWidget(self.splitter)
		self.stackedWidget.setObjectName(_fromUtf8("stackedWidget"))
		self.pageMain = QtGui.QWidget()
		self.pageMain.setObjectName(_fromUtf8("pageMain"))
		self.stackedWidget.addWidget(self.pageMain)
		# Slots
		self.bind()
		# Logic
		self.server = None
		self.formFactory = None
		self.currentForm = None
		self.currentItem = None
		# Container
		self.container = None
	def bind(self):
		QtCore.QObject.connect(self,QtCore.SIGNAL("actionDownload(PyQt_PyObject)"),self.slotDownload)
		# Explorer
		QtCore.QObject.connect(self.explorer,QtCore.SIGNAL("updatedTree()"),self.slotUpdatedTree)
		QtCore.QObject.connect(self.explorer,QtCore.SIGNAL("selected(QStandardItem,PyQt_PyObject)"),self.slotSelected)
		QtCore.QObject.connect(self.explorer,QtCore.SIGNAL("actionDownload(PyQt_PyObject)"),self.slotDownload)
# ############################## PROPERTIES ###################################
	def setContainer(self,c):
		self.container = c
	def setServer(self,server):
		self.server = server
		self.explorer.setServer(server)
	def getServer(self):
		return self.server
	def setFormFactory(self,formFactory):
		self.formFactory = formFactory
		self.explorer.setFormFactory(formFactory)
		# Update UI
		# Cleaning stackedWidget
		for i in range(self.stackedWidget.count()):
			if i==0:
				continue
			self.stackedWidget.removeWidget( self.stackedWidget.widget(i) )
		# Form Cache
		self.forms={}
		self.formWidgets = {}
		formList = self.formFactory.getAllClassnames()
		for _f in formList:
			if _f in ['default','FBanned','FLog','FObject'
						,'FProjectPeopleRole','FProjectCompanyRole','FTodoTipo','FUserGroupAssociation']:
				continue
			if _f.find("Filter")>=0:
				continue
			myform = self.formFactory.getInstance(_f, dbmgr=self.server)
			if isinstance(myform,FAssociation):
				continue
			w = myform.render(self.stackedWidget)
			self.stackedWidget.addWidget(w)
			QtCore.QObject.connect(myform,QtCore.SIGNAL("clickedButton(PyQt_PyObject)"),self.slotFormClickedButton)
			self.forms[_f] = myform
			self.formWidgets[_f] = w
	def getName(self):
		d = self.server.getConnectionProvider().getDBType()
		u = "nobody"
		if not self.server.getDBEUser() is None:
			u = self.server.getDBEUser().getValue('login')
		h = "%s" % self.server.getConnectionProvider().getHost()
		if d=="SQLite":
			h = ( "%s" % self.server.getConnectionProvider().getDB() ).split( os.path.sep )[-1]
		if h.find("://")>0:
			_url = h.split("://")
			if len(_url)>1:
				h = "%s:%s" % ( _url[0],_url[1].split("/")[0] )
		return "%s@%s" % (u,h)
# ############################## PREFERENCES ###################################
	def getPref(self,section,option,default=None):
		if self.container is None:
			return default
		return self.container.getPref(section,option,default)
	def setPref(self,section,option,valore):
		if self.container is None:
			return
		self.container.setPref(section,option,valore)
# ############################## ... ###################################
	def Alert(self,msg, eccezione=None):
		if self.container is None:
			return
		self.container.Alert(msg,eccezione)
	def Confirm(self,msg):
		if self.container is None:
			return True
		return self.container.Confirm(msg)
	def SetStatusText(self,msg):
		if self.container is None:
			return
		self.container.SetStatusText(msg)
	def SetProgress(self,perc):
		if self.container is None:
			return
		self.container.SetProgress(perc)
# ############################## UI ###################################
	def updateObject(self,myitem,mydbe):
		myobj = mydbe
		# Load full object definition
		if mydbe.getTypeName()=="DBEObject":
			myobj = self.server.fullObjectById( mydbe.getValue('id') )
		# dbe 2 form
		try:
			myformname = self.formFactory.dbename2classname[myobj.getTypeName()]
			_form = self.forms[myformname]
			self.currentForm = _form
			self.currentItem = myitem
			self.stackedWidget.setCurrentWidget( self.formWidgets[myformname] )
			_form.clean()
			_form.setValues( myobj.getValuesDictionary() )
		except Exception,e:
			self.Alert("Error while executing updateObject: %s"%e,e)
# ############################## LOGIC ###################################
	def runSearchFolder(self,name,mydbetype='DBEObject'):
		self.explorer.runSearchFolder(name,mydbetype)
	def _form2item(self,item):
		if item is None:
			return
		values = self.currentForm.getValues()
		mydata = item.data().toPyObject()
		mydata.setValuesDictionary(values)
		item.setData(mydata)
		return item
	def actionClose(self,dbe):
		self.stackedWidget.setCurrentWidget( self.pageMain )
	def actionSave(self,dbe):
		mydbe = dbe
		if mydbe.isNew():
			mydbe = self.server.insert(mydbe)
		else:
			mydbe = self.server.update(mydbe)
		if mydbe is None:
			self.SetStatusText("Save: errors.")
			return None
		# Item update
		if self.currentItem is None:
			return
		myitem = self.currentItem
		myitem.setText( u"%s" % mydbe.getValue('name') )
		myitem.setData( mydbe )
		desc = mydbe.getValue('description')
		if desc>'':
			myitem.setToolTip( desc )
		self.updateObject(myitem,mydbe)
		self.SetStatusText("Save: OK")
	def actionDelete(self,dbe):
		msg = QtGui.QApplication.translate("ServerWidget", "Confirm Delete?", None, QtGui.QApplication.UnicodeUTF8)
		if not self.Confirm( msg ):
			return
		myitem = self.currentItem
		if myitem is None:
			return
		myitem = self._form2item(myitem)
		mydbe = dbe
		if mydbe.isNew():
			mydbe = None
		else:
			mydbe = self.server.delete(mydbe)
		if not mydbe is None and not mydbe.isDeleted():
			self.SetStatusText("Delete: errors.")
			return
		myparent = myitem.parent()
		self.explorer.removeItem(myitem)
		mydbe = myparent.data().toPyObject()
		self.updateObject(myparent,mydbe)
		self.SetStatusText("Delete: OK")
	def actionDownload(self,dbe):
		currentDir = self.getPref("Main","download_path","%s%sDownloads"%(os.path.expanduser("~"),os.path.sep))
		fd = QtGui.QFileDialog(self,"Select destination...", "%s%s%s" % (currentDir,os.path.sep,dbe.getValue('filename')))
		fd.setAcceptMode(QtGui.QFileDialog.AcceptSave)
		if not fd.exec_():
			return
		currentDir = "%s" % ( fd.directory().path() )
		self.setPref("Main","download_path",currentDir)
		try:
			local_filename = self.server.downloadFile( dbe.getValue('id'), currentDir )
			self.Alert("Download complete!")
		except Exception,e:
			self.Alert("Error while downloading!",e)
# ############################## SLOTS ###################################
	def slotUpdatedTree(self):
		self.stackedWidget.setCurrentWidget( self.pageMain )
	def slotSelected(self,item,dbe):
		self.updateObject(item,dbe)
	def slotFormClickedButton(self,s):
		sender = self.sender()
		mydbe = sender.getDBE(self.server)
		mydbe.setValuesDictionary( sender.getValues() )
		tmp = s.split("_")
		nomeform=''
		if len(tmp)>1:
			nomeform=tmp[1]
		if len(tmp)>2:
			nomeazione="_".join(tmp[2:])
		customAction = "action%s%s" % (nomeazione[0].upper(),nomeazione[1:])
		if customAction=='actionDownload':
			self.emit(QtCore.SIGNAL("actionDownload(PyQt_PyObject)"),mydbe)
			return
		try:
			getattr(self,customAction)(mydbe)
		except Exception,e:
			self.Alert("Error while executing '%s'." % nomeazione,e)
	def slotDownload(self,dbe):
		self.actionDownload(dbe)



# -*- coding: utf-8 -*-
from rprj.formulator.qtgui import *

formschema_type_list={}

class FUser(FMasterDetail):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FMasterDetail.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		self.addField('',-1,'id', \
			FNumber('id','ID','','','formtable') )
		self.addField('',-1,'login', \
			FString('login','Login','Inserire la login',50,255,'','formtable') )
		self.addField('',-1,'pwd', \
			FPassword('pwd','Password','Inserire la login',50,255,'','formtable') )
		self.addField('',-1,'pwd_salt', \
			FList('pwd_salt','Password encrypting','Password encrypt saalt',255,40,'0','formtable',{ '':'-', 'md5':'MD5', 'sha1':'SHA1'},1,False) )
		self.addField('',-1,'fullname', \
			FString('fullname','Full Name','Inserire la login',50,255,'','formtable') )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('group_id')
		self.addField('',-1,'group_id', \
			FKField('group_id','Group','Active group.','-4','formtable',_mydbe,_tmpFK,['name'],'FGroup','select',' - ',1,False) )
		# Details
		self.addDetail("FUserGroupAssociation")
		self.addDetail("FPeople")
	def getDetailIcon(self):
		return "icons/user.png"
	def getDetailTitle(self):
		return 'User'
	def getDetailColumnNames(self):
		return ['login','pwd','pwd_salt','fullname','group_id']
	def getFilterForm(self):
		return FUser()
	def getFilterFields(self):
		return ['login','fullname','group_id']
	def getListTitle(self):
		return 'Users'
	def getListColumnNames(self):
		return ['id','login','fullname','group_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEUser")()
		try:
			return DBEUser()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		return self.getValue('login')
formschema_type_list["FUser"]=FUser

class FGroup(FMasterDetail):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FMasterDetail.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'id', \
			FNumber('id','ID','','','formtable') )
		self.addField('',-1,'name', \
			FString('name','Name','Nome del gruppo.',50,255,'','formtable') )
		self.addField('',-1,'description', \
			FTextArea('description','Description','Descrizione del gruppo.',255,'','formtable',50,5,True) )
		# Details
		self.addDetail("FUserGroupAssociation")
	def getDetailIcon(self):
		return "icons/group_16x16.gif"
	def getDetailTitle(self):
		return 'Group'
	def getDetailColumnNames(self):
		return ['name','description']
	def getFilterForm(self):
		return FGroup()
	def getFilterFields(self):
		return ['name','description']
	def getListTitle(self):
		return 'Groups'
	def getListColumnNames(self):
		return ['id','name','description']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEGroup")()
		try:
			return DBEGroup()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		return self.getValue('name')
formschema_type_list["FGroup"]=FGroup

class FUserGroupAssociation(FAssociation):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FAssociation.__init__(self,nome,azione,metodo,enctype,dbmgr)
	def getDetailTitle(self):
		return 'Association User-Group'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEUserGroup")()
		try:
			return DBEUserGroup()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		raise Exception("TODO")
formschema_type_list["FUserGroupAssociation"]=FUserGroupAssociation

class FLog(FMasterDetail):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FMasterDetail.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		self.addField('',-1,'ip', \
			FString('ip','IP','',16,16,'','formtable') )
		self.addField('',-1,'data', \
			FDateTime('data','Date','','2012-01-29','formtable',True,False) )
		self.addField('',-1,'ora', \
			FDateTime('ora','Hour','',dbmgr.getTodayString(),'formtable',False,True) )
		self.addField('',-1,'count', \
			FNumber('count','Count','','','formtable') )
		self.addField('',-1,'url', \
			FString('url','URL','',50,255,'','formtable') )
		self.addField('',-1,'note', \
			FString('note','Note','',50,255,'','formtable') )
		self.addField('',-1,'note2', \
			FTextArea('note2','Note 2','',255,'','formtable',50,5,True) )
	def getDetailIcon(self):
		return "icons/text-x-log.png"
	def getDetailTitle(self):
		return 'Log'
	def getDetailColumnNames(self):
		return ['data','ora','count','url','note','note2']
	def getFilterFields(self):
		return ['ip','data','count','url','note']
	def getListTitle(self):
		return 'Log'
	def getListColumnNames(self):
		return ['data','ora','count','ip','url','note']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBELog")()
		try:
			return DBELog()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		return "%s - %s" % (self.getValue('ip'),self.getValue('url'))
formschema_type_list["FLog"]=FLog

class FLogFilter(FLog):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FLog.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'from_ip', \
			FString('from_ip','IP >=','',16,16,'','formtable') )
		self.addField('',-1,'to_ip', \
			FString('to_ip','IP <=','',16,16,'','formtable') )
		self.addField('',-1,'from_data', \
			FDateTime('from_data','Date >=','Date from','2012-01-29','formtable',True,False) )
		self.addField('',-1,'to_data', \
			FDateTime('to_data','Date <=','Date to','2012-01-29','formtable',True,False) )
	def getDetailIcon(self):
		return "icons/text-x-log.png"
	def getDetailTitle(self):
		return 'Log Filter'
	def getDetailColumnNames(self):
		return ['ora','count','url','note','note2']
	def getFilterFields(self):
		return ['from_ip','to_ip','from_data','to_data','url','note']
	def getListTitle(self):
		return 'Log'
	def getListColumnNames(self):
		return ['data','ora','count','ip','url','note']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBELog")()
		try:
			return DBELog()
		except Exception,e:
			return None
formschema_type_list["FLogFilter"]=FLogFilter

class FObject(FMasterDetail):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FMasterDetail.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		self.addField('',-1,'id', \
			FUuid('id','ID','',16,0,'formtable','') )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('creator')
		self.addField('',-1,'creator', \
			FKField('creator','Created by','Creatore','','formtable',_mydbe,_tmpFK,['login'],'FUser','readonly',' - ',1,False) )
		self.addField('',-1,'creation_date', \
			FDateTimeReadOnly('creation_date','Created on','Creato il',dbmgr.getTodayString(),'formtable',True,True) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('last_modify')
		self.addField('',-1,'last_modify', \
			FKField('last_modify','Modified by','Modificato da','','formtable',_mydbe,_tmpFK,['login'],'FUser','readonly',' - ',1,False) )
		self.addField('',-1,'last_modify_date', \
			FDateTimeReadOnly('last_modify_date','Modified on','Ultima modifica',dbmgr.getTodayString(),'formtable',True,True) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('deleted_by')
		self.addField('',-1,'deleted_by', \
			FKField('deleted_by','Deleted by','Deleted by','','formtable',_mydbe,_tmpFK,['login'],'FUser','readonly',' - ',1,False) )
		self.addField('',-1,'deleted_date', \
			FDateTimeReadOnly('deleted_date','Deleted on','Deleted on','0000-00-00 00:00:00','formtable',True,True) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('father_id')
		self.addField('',-1,'father_id', \
			FKObjectField('father_id','Parent','Padre','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('',-1,'name', \
			FString('name','Name','Nome del gruppo.',50,255,'','formtable') )
		self.addField('',-1,'description', \
			FTextArea('description','Description','Descrizione del gruppo.',255,'','formtable',-1,-1,True) )
			#FTextArea('description','Description','Descrizione del gruppo.',255,'','formtable',50,5,True) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('owner')
		self.addField('_permission',-1,'owner', \
			FKField('owner','Owner','Proprietario.','','formtable',_mydbe,_tmpFK,['login'],'FUser','select',' - ',1,False) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('group_id')
		self.addField('_permission',-1,'group_id', \
			FKField('group_id','Group','Active group.','','formtable',_mydbe,_tmpFK,['name'],'FGroup','select',' - ',1,False) )
		self.addField('_permission',-1,'permissions', \
			FPermissions('permissions','Permissions','Unix style',9,9,'rwx------','formtable') )
	def getDetailTitle(self):
		return 'Object'
	def getDetailColumnNames(self):
		return [\
			'creator','creation_date','last_modify','last_modify_date','deleted_by','deleted_date',\
			'father_id',
			'name','description','owner','group_id','permissions',\
			]
	def getDetailReadOnlyColumnNames(self):
		return ['id','creator','creation_date','last_modify','last_modify_date','deleted_by','deleted_date',]
	def getFilterFields(self):
		return ['name','description','owner','group_id',]
	def getListTitle(self):
		return 'Objects'
	def getDecodeGroupNames(self):
		ret = FMasterDetail.getDecodeGroupNames(self)
		#ret['']='Info'
		ret['_permission']='Permissions'
		return ret
	def getGroupNames(self):
		ret = [k for k in FMasterDetail.getGroupNames(self) if not k in ['','_permission'] ]
		ret.append('')
		ret.append('_permission')
		return ret
	def getListColumnNames(self):
		return ['id','name','description']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEObject")()
		try:
			return DBEObject()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		return self.getValue('name')
formschema_type_list["FObject"]=FObject

class FCompany(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		self.addField('contact',-1,'p_iva', \
			FString('p_iva','P.IVA','Partita IVA',50,255,'','formtable') )
		self.addField('phone',-1,'phone', \
			FString('phone','Phone','Telefono',50,255,'','formtable') )
		self.addField('phone',-1,'fax', \
			FString('fax','Fax','Fax',50,255,'','formtable') )
		self.addField('web',-1,'email', \
			FString('email','Email','Email',50,255,'','formtable') )
		self.addField('web',-1,'url', \
			FString('url','URL','Sito web',50,255,'','formtable') )
		self.addField('address',-1,'street', \
			FString('street','Street','Via',50,255,'','formtable') )
		self.addField('address',-1,'zip', \
			FString('zip','ZIP','CAP',10,255,'','formtable') )
		self.addField('address',-1,'city', \
			FString('city','City','Città.',50,255,'','formtable') )
		self.addField('address',-1,'state', \
			FString('state','State','Provincia.',20,255,'','formtable') )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_countrylist_id')
		self.addField('address',-1,'fk_countrylist_id', \
			FKField('fk_countrylist_id','Country','Nazione.','82','formtable',_mydbe,_tmpFK,['Common_Name'],'','select',' - ',1,False) )
		# Details
		self.addDetail("FFolder")
		self.addDetail("FProjectCompany")
		self.addDetail("FPeople")
		self.addDetail("FPage")
		self.addDetail("FLink")
		self.addDetail("FNote")
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','contact',0)
		self.addToGroup('description','contact')
		myfield = self.getField('description')
		myfield.height = 5
	def getDetailIcon(self):
		return "icons/company_16x16.gif"
	def getDetailTitle(self):
		return 'Company'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["p_iva","street","zip","city","state","fk_countrylist_id","phone","fax","email","url"]:
			ret.append(x)
		return ret
	def getFilterForm(self):
		return FCompany()
	def getFilterFields(self):
		return ['name','p_iva','zip','city','fk_countrylist_id']
	def getListTitle(self):
		return 'Companies'
	def getGroupNames(self):
		return ['contact','address','phone','web','','_permission']
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['contact']='Company'
		ret['phone']='Phone'
		ret['address']='Address'
		ret['web']='Web'
		return ret
	def getListColumnNames(self):
		return ['name','phone','city','fk_countrylist_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBECompany")()
		try:
			return DBECompany()
		except Exception,e:
			return None
formschema_type_list["FCompany"]=FCompany

class FPeople(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_users_id')
		self.addField('contact',-1,'fk_users_id', \
			FKField('fk_users_id','User','Utente.','','formtable',_mydbe,_tmpFK,['login'],'FUser','select',' - ',1,False) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_companies_id')
		self.addField('contact',-1,'fk_companies_id', \
			FKField('fk_companies_id','Company','Organizzazione.','','formtable',_mydbe,_tmpFK,['name','state'],'FCompany','select',' - ',1,False) )
		self.addField('contact',-1,'codice_fiscale', \
			FString('codice_fiscale','Codice Fiscale','Codice Fiscale',50,255,'','formtable') )
		self.addField('contact',-1,'p_iva', \
			FString('p_iva','P.IVA','Partita IVA',50,255,'','formtable') )
		self.addField('phone',-1,'office_phone', \
			FString('office_phone','Office Phone','Telefono Ufficio',50,255,'','formtable') )
		self.addField('phone',-1,'mobile', \
			FString('mobile','Mobile','Cellulare',50,255,'','formtable') )
		self.addField('phone',-1,'phone', \
			FString('phone','Phone','Telefono',50,255,'','formtable') )
		self.addField('phone',-1,'fax', \
			FString('fax','Fax','Fax',50,255,'','formtable') )
		self.addField('address',-1,'street', \
			FString('street','Street','Via',50,255,'','formtable') )
		self.addField('address',-1,'zip', \
			FString('zip','ZIP','CAP',10,255,'','formtable') )
		self.addField('address',-1,'city', \
			FString('city','City','Città.',50,255,'','formtable') )
		self.addField('address',-1,'state', \
			FString('state','State','Provincia.',20,255,'','formtable') )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_countrylist_id')
		self.addField('address',-1,'fk_countrylist_id', \
			FKField('fk_countrylist_id','Country','Nazione.','82','formtable',_mydbe,_tmpFK,['Common_Name'],'','select',' - ',1,False) )
		self.addField('web',-1,'email', \
			FString('email','Email','Email',50,255,'','formtable') )
		self.addField('web',-1,'url', \
			FString('url','URL','Sito web',50,255,'','formtable') )
		# Details
		self.addDetail("FFolder")
		self.addDetail("FProjectPeople")
		self.addDetail("FPage")
		self.addDetail("FLink")
		self.addDetail("FNote")
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','contact',0)
		self.addToGroup('description','contact')
		myfield = self.getField('description')
		myfield.height = 5
	def getDetailIcon(self):
		return "icons/people.png"
	def getDetailTitle(self):
		return 'Person'
	def getDetailColumnNames(self):
		return ['name','fk_users_id','fk_companies_id','codice_fiscale','p_iva','street','zip','city','state','fk_countrylist_id','phone','office_phone','mobile','fax','email','url','owner','group_id','permissions']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date']
	def getFilterForm(self):
		return FPeople()
	def getFilterFields(self):
		return ['name','zip','city','fk_countrylist_id','fk_companies_id']
	def getListTitle(self):
		return 'People'
	def getGroupNames(self):
		return ['contact','address','phone','web','','_permission']
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['contact']='Person'
		ret['phone']='Phone'
		ret['address']='Address'
		ret['web']='Web'
		return ret
	def getListColumnNames(self):
		return ['name','phone','email','fk_companies_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEPeople")()
		try:
			return DBEPeople()
		except Exception,e:
			return None
formschema_type_list["FPeople"]=FPeople

class FEvent(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('event',-1,'start_date', \
			FDateTime('start_date','Start','Start',dbmgr.getTodayString(),'formtable',True,True) )
		self.addField('event',-1,'end_date', \
			FDateTime('end_date','End','End',dbmgr.getTodayString(),'formtable',True,True) )
		self.addField('event',-1,'all_day', \
			FList('all_day','All day','All day event',255,40,'0','formtable',{ '0':'No', '1':'Yes'},1,False) )
		self.addField('event',-1,'url', \
			FString('url','Url','Url',50,255,'','formtable') )
		self.addField('event',-1,'category', \
			FString('category','Category','Category',50,255,'','formtable') )
		# Details
		self.addDetail("FPeople")
		self.addDetail("FLink")
		self.addDetail("FFile")
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','event',3)
		self.addToGroup('description','event',4)
	def getDetailIcon(self):
		return "icons/event_16x16.png"
	def getDetailTitle(self):
		return 'Event'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["fk_obj_id","start_date","end_date","all_day","url","category","alarm","alarm_unit","alarm_minute","before_event","recurrence","recurrence_type"]:
			ret.append(x)
		return ret
	def getFilterFields(self):
		return ['name','father_id']
	def getListTitle(self):
		return 'Events'
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['event']='Event'
		ret['recurrence']='Recurrence'
		ret['alarm']='Alarm'
		return ret
	def getListColumnNames(self):
		return ['start_date','end_date','name','father_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEEvent")()
		try:
			return DBEEvent()
		except Exception,e:
			return None
formschema_type_list["FEvent"]=FEvent

class FEventFilter(FEvent):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FEvent.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'from_start_date', \
			FDateTime('from_start_date','Date >=','Date from','2012-01-29','formtable',True,False) )
		self.addField('',-1,'to_start_date', \
			FDateTime('to_start_date','Date <=','Date to','','formtable',True,False) )
		# Details
		self.addDetail("FPeople")
		self.addDetail("FLink")
		self.addDetail("FFile")
	def getDetailIcon(self):
		return "icons/event_16x16.png"
	def getDetailTitle(self):
		return 'Event Filter'
	def getDetailColumnNames(self):
		return ['creation_date','creator','father_id','name','description','fk_obj_id','owner','group_id','permissions','start_date','end_date','all_day','url','category','alarm','alarm_unit','alarm_minute','before_event','recurrence','recurrence_type']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date']
	def getFilterFields(self):
		return ['name','from_start_date','to_start_date']
	def getListTitle(self):
		return 'Events'
	def getListColumnNames(self):
		return ['start_date','end_date','name','father_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEEvent")()
		try:
			return DBEEvent()
		except Exception,e:
			return None
# FIXME formschema_type_list["FEventFilter"]=FEventFilter

class FFile(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('file',-1,'path', \
			FString('path','Path','Percorso',50,255,'','formtable') )
		self.addField('file',-1,'filename', \
			FFileField('filename','File name','Nome del file',20,255,'','formtable') )
		self.addField('file',-1,'checksum', \
			FString('checksum','Checksum','Checksum SHA1',40,40,'','formtable') )
		self.addField('file',-1,'mime', \
			FString('mime','Mime type','Mime type',40,40,'','formtable') )
		self.addField('file',-1,'alt_link', \
			FString('alt_link','Alternative Link','Link alternativo',50,255,'','formtable') )
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','file',0)
		self.addToGroup('description','file',1)
	def getDetailIcon(self):
		return "icons/file_16x16.gif"
	def getDetailTitle(self):
		return 'File'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["path","filename","checksum","mime","alt_link"]:
			ret.append(x)
		return ret
	def getDetailReadOnlyColumnNames(self):
		ret = FObject.getDetailReadOnlyColumnNames(self)
		for x in ["checksum","mime"]:
			ret.append(x)
		return ret
	def getFilterForm(self):
		return FFile()
	def getFilterFields(self):
		return ['father_id','name','description']
	def getListTitle(self):
		return 'Files'
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['file']='File'
		return ret
	def getListColumnNames(self):
		return ['name','father_id','mime','path','filename']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEFile")()
		try:
			return DBEFile()
		except Exception,e:
			return None
	def getActions(self):
		ret = FObject.getActions(self)
		ret['download'] = ['Download','_do.php','icons/download.png','Download']
		return ret
	def isImage(self):
		_mime = self.getValue('mime')
		return _mime>'' and _mime[0:5]=='image'
formschema_type_list["FFile"]=FFile

class FFolder(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('folder',-1,'childs_sort_order', \
			FChildSort('childs_sort_order','Sort order','Ordinamento dei figli diretti',255,40,'','formtable',{},10,True) )
		# Details
		self.addDetail("FFolder")
		self.addDetail("FPage")
		self.addDetail("FNews")
		self.addDetail("FLink")
		self.addDetail("FNote")
		self.addDetail("FEvent")
		self.addDetail("FFile")
		self.addDetail("FMail")
		self.addDetail("FTodo")
		self.addDetail("FTimetrack")
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','folder',0)
		self.addToGroup('description','folder',1)
	def getDetailIcon(self):
		return "icons/folder_16x16.gif"
	def getDetailTitle(self):
		return 'Folder'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["fk_obj_id","childs_sort_order"]:
			ret.append(x)
		return ret
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['folder']='Folder'
		return ret
	def getFilterForm(self):
		return FFolder()
	def getFilterFields(self):
		return ['name','fk_obj_id','father_id']
	def getListTitle(self):
		return 'Folders'
	def getListColumnNames(self):
		return ['name','description','father_id','fk_obj_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEFolder")()
		try:
			return DBEFolder()
		except Exception,e:
			return None
formschema_type_list["FFolder"]=FFolder

class FLink(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('link',-1,'href', \
			FString('href','Href','Url destinazione',50,255,'','formtable') )
		self.addField('link',-1,'target', \
			FString('target','Target','Frame target.',50,255,'','formtable') )
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','link',0)
		self.addToGroup('description','link')
		myfield = self.getField('description')
		myfield.height = 5
	def getDetailIcon(self):
		return "icons/link_16x16.gif"
	def getDetailTitle(self):
		return 'Link'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["href","target","fk_obj_id"]:
			ret.append(x)
		return ret
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['link']='Link'
		return ret
	def getFilterForm(self):
		return FLink()
	def getFilterFields(self):
		return ['fk_obj_id','name','href']
	def getListTitle(self):
		return 'Links'
	def getListColumnNames(self):
		return ['creation_date','fk_obj_id','name','href']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBELink")()
		try:
			return DBELink()
		except Exception,e:
			return None
formschema_type_list["FLink"]=FLink

class FNote(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','note',0)
		self.addToGroup('description','note')
	def getDetailIcon(self):
		return "icons/note_16x16.gif"
	def getDetailTitle(self):
		return 'Note'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["fk_obj_id"]:
			ret.append(x)
		return ret
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['note']='Note'
		return ret
	def getFilterForm(self):
		return FNote()
	def getFilterFields(self):
		return ['name','fk_obj_id']
	def getListTitle(self):
		return 'Notes'
	def getListColumnNames(self):
		return ['creation_date','name','fk_obj_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBENote")()
		try:
			return DBENote()
		except Exception,e:
			return None
formschema_type_list["FNote"]=FNote

class FPage(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('html',-1,'html', \
			FHtml('html','Html','Contenuto html.',255,'','formtable',50,50,True) )
		self.addField('html',-1,'language', \
			FLanguage('language','Language','Page language',50,255,'en_US','formtable') )
		# Details
		self.addDetail("FPage")
		self.addDetail("FLink")
		self.addDetail("FFile")
		self.addDetail("FEvent")
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','html',0)
		self.addToGroup('description','html',1)
		myfield = self.getField('description')
		myfield.height = 2
	def getDetailIcon(self):
		return "icons/page_16x16.gif"
	def getDetailTitle(self):
		return 'Page'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["language","html","fk_obj_id"]:
			ret.append(x)
		return ret
	#def getDetailReadOnlyColumnNames(self):
		#return ['creator','creation_date','last_modify','last_modify_date','deleted_by','deleted_date',]
	def getFilterForm(self):
		return FPage()
	def getFilterFields(self):
		return ['name','fk_obj_id','father_id']
	def getListTitle(self):
		return 'Pages'
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['html']='Page'
		return ret
	def getListColumnNames(self):
		return ['creation_date','name','fk_obj_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEPage")()
		try:
			return DBEPage()
		except Exception,e:
			return None
formschema_type_list["FPage"]=FPage

class FNews(FPage):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FPage.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Details
		self.addDetail("FLink")
		self.addDetail("FFile")
		self.addDetail("FEvent")
		self.addDetail("FNews")
	def getDetailIcon(self):
		return "icons/news.png"
	def getDetailTitle(self):
		return 'News'
	def getDecodeGroupNames(self):
		ret = FPage.getDecodeGroupNames(self)
		ret['html']='News'
		return ret
	def getFilterForm(self):
		return FNews()
	def getFilterFields(self):
		return ['name','fk_obj_id','father_id']
	def getListTitle(self):
		return 'News'
	def getListColumnNames(self):
		return ['creation_date','name','fk_obj_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBENews")()
		try:
			return DBENews()
		except Exception,e:
			return None
formschema_type_list["FNews"]=FNews


# #### Projects: start.

class FProject(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Details
		self.addDetail("FFolder")
		self.addDetail("FProjectProject")
		self.addDetail("FProjectCompany")
		self.addDetail("FProjectPeople")
		self.addDetail("FTodo")
		self.addDetail("FTimetrack")
		self.addDetail("FPage")
		self.addDetail("FLink")
		self.addDetail("FNote")
		self.addDetail("FEvent")
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','project',0)
		self.addToGroup('description','project')
	def getDetailIcon(self):
		return "icons/project_16x16.gif"
	def getDetailTitle(self):
		return 'Project'
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['project']='Project'
		return ret
	def getListTitle(self):
		return 'Projects'
	def getListColumnNames(self):
		return ['name','description']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEProject")()
		try:
			return DBEProject()
		except Exception,e:
			return None
formschema_type_list["FProject"]=FProject

class FProjectCompany(FAssociation):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FAssociation.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('projects_companies_role_id')
		self.addField('',-1,'projects_companies_role_id', \
			FKField('projects_companies_role_id','Role','Ruolo.','','formtable',_mydbe,_tmpFK,['projects_companies_role_id'],'FCompany','select',' - ',1,False) )
	def getDetailTitle(self):
		return 'Association Project-Company'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEProjectCompany")()
		try:
			return DBEProjectCompany()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		raise Exception("TODO")
formschema_type_list["FProjectCompany"]=FProjectCompany

class FProjectCompanyRole(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'order_position', \
			FString('order_position','Order','Ordine',50,255,'','formtable') )
	def getDetailTitle(self):
		return 'Project-Company Role'
	def getDetailColumnNames(self):
		return ['name','description','order_position']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date']
	def getListTitle(self):
		return 'Project-Company Roles'
	def getListColumnNames(self):
		return ['order_position','name','description']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEProjectCompanyRole")()
		try:
			return DBEProjectCompanyRole()
		except Exception,e:
			return None
formschema_type_list["FProjectCompanyRole"]=FProjectCompanyRole

class FProjectPeople(FAssociation):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FAssociation.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('projects_people_role_id')
		self.addField('',-1,'projects_people_role_id', \
			FKField('projects_people_role_id','Role','Ruolo.','','formtable',_mydbe,_tmpFK,['projects_people_role_id'],'FPeople','select',' - ',1,False) )
	def getDetailTitle(self):
		return 'Association Project-People'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEProjectPeople")()
		try:
			return DBEProjectPeople()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		raise Exception("TODO")
formschema_type_list["FProjectPeople"]=FProjectPeople

class FProjectPeopleRole(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'order_position', \
			FString('order_position','Order','Ordine',50,255,'','formtable') )
	def getDetailTitle(self):
		return 'Project-People Role'
	def getDetailColumnNames(self):
		return ['name','description','order_position']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date']
	def getListTitle(self):
		return 'Project-People Roles'
	def getListColumnNames(self):
		return ['order_position','name','description']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEProjectPeopleRole")()
		try:
			return DBEProjectPeopleRole()
		except Exception,e:
			return None
formschema_type_list["FProjectPeopleRole"]=FProjectPeopleRole

class FProjectProject(FAssociation):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FAssociation.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('projects_projects_role_id')
		self.addField('',-1,'projects_projects_role_id', \
			FKField('projects_projects_role_id','Role','Ruolo.','','formtable',_mydbe,_tmpFK,['projects_projects_role_id'],'FProject','select',' - ',1,False) )
	def getDetailTitle(self):
		return 'Association Project-Project'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEProjectProject")()
		try:
			return DBEProjectProject()
		except Exception,e:
			return None
	def getShortDescription(self,dbmgr=None):
		raise Exception("TODO")
formschema_type_list["FProjectProject"]=FProjectProject

class FTimetrack(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_obj_id')
		self.addField('',-1,'fk_obj_id', \
			FKObjectField('fk_obj_id','Linked to','Collegata a','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_progetto')
		self.addField('',-1,'fk_progetto', \
			FKObjectField('fk_progetto','Project','Progetto','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		self.addField('timetrack',-1,'dalle_ore', \
			FDateTime('dalle_ore','From','Dalle Ore',dbmgr.getTodayString(),'formtable',True,True) )
		self.addField('timetrack',-1,'alle_ore', \
			FDateTime('alle_ore','To','Alle Ore','','formtable',True,True) )
			#FDateTime('alle_ore','To','Alle Ore',dbmgr.getTodayString(),'formtable',True,True) )
		self.addField('timetrack',-1,'ore_intervento', \
			FDateTime('ore_intervento','Hours of Work','Ore Intervento','','formtable',False,True) )
		self.addField('timetrack',-1,'ore_viaggio', \
			FDateTime('ore_viaggio','Hours of Travel','Ore Viaggio','','formtable',False,True) )
		self.addField('timetrack',-1,'km_viaggio', \
			FNumber('km_viaggio','KM Travels','KM Viaggio','','formtable') )
		self.addField('timetrack',-1,'luogo_di_intervento', \
			FList('luogo_di_intervento','Luogo di intervento','Luogo di intervento',255,40,'0','formtable',{ '':'', '0':'Office', '1':'From office (via ssh/telnet/ecc.)', '2':'On site'},1,False) )
		self.addField('timetrack',-1,'stato', \
			FList('stato','State','Stato',255,40,'0','formtable',{ '':'', '0':'da fatturare', '1':'non fatturare', '2':'fatturato', '3':'assistenza'},1,False) )
		self.addField('timetrack',-1,'costo_per_ora', \
			FNumber('costo_per_ora','Cost per hour','Cost per hour','0','formtable') )
		self.addField('timetrack',-1,'costo_valuta', \
			FString('costo_valuta','Currency','Currency',50,255,'euro','formtable') )
		# Beautifying
		self.removeFromGroup('name','')
		self.removeFromGroup('description','')
		self.addToGroup('name','timetrack',0)
		self.addToGroup('description','timetrack',1)
		myfield = self.getField('description')
		myfield.height = 10
	def getDetailIcon(self):
		return "icons/timetrack_16x16.gif"
	def getDetailTitle(self):
		return 'Timetrack'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		for x in ["fk_progetto","fk_obj_id","dalle_ore","alle_ore","ore_intervento","ore_viaggio","km_viaggio","luogo_di_intervento","stato","costo_per_ora","costo_valuta"]:
			ret.append(x)
		return ret
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['timetrack']='Timetrack'
		return ret
	def getFilterForm(self):
		return FTimetrack()
	def getFilterFields(self):
		return ['fk_progetto','name','fk_obj_id','luogo_di_intervento','stato']
	def getListTitle(self):
		return 'Timetracks'
	def getListColumnNames(self):
		return ['fk_progetto','stato','dalle_ore','name','ore_intervento','fk_obj_id']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBETimetrack")()
		try:
			return DBETimetrack()
		except Exception,e:
			return None
formschema_type_list["FTimetrack"]=FTimetrack

class FTodo(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		self.addField('todo',-1,'priority', \
			FNumber('priority','Priority','Priorita','','formtable') )
		self.addField('todo',-1,'data_segnalazione', \
			FDateTime('data_segnalazione','Reported on','Data segnalazione',dbmgr.getTodayString(),'formtable',True,True) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_segnalato_da')
		self.addField('todo',-1,'fk_segnalato_da', \
			FKField('fk_segnalato_da','Reporter','Segnalato da','','formtable',_mydbe,_tmpFK,['name'],'FPeople','select',' - ',1,False) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_cliente')
		self.addField('todo',-1,'fk_cliente', \
			FKField('fk_cliente','Customer','Cliente','','formtable',_mydbe,_tmpFK,['name'],'FCompany','select',' - ',1,False) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_progetto')
		self.addField('todo',-1,'fk_progetto', \
			FKObjectField('fk_progetto','Project','Progetto','','formtable',_mydbe,_tmpFK,['name'],'','select',' - ',1,False) )
		_tmpFK=None
		if not _mydbe is None:
			_tmpFK=_mydbe.getFKDefinition('fk_tipo')
		self.addField('todo',-1,'fk_tipo', \
			FKField('fk_tipo','Kind','Tipo','','formtable',_mydbe,_tmpFK,['name'],'FTodoTipo','select',' - ',1,False) )
		self.addField('todo',-1,'stato', \
			FPercent('stato','State (%)','Stato di avanzamento %','','formtable') )
		self.addField('todo',-1,'descrizione', \
			FHtml('descrizione','Description','Descrizione del todo.',255,'','formtable',-1,5,True) )
		self.addField('todo',-1,'intervento', \
			FHtml('intervento','Resolution','Intervento eseguito.',255,'','formtable',-1,5,True) )
		self.addField('todo',-1,'data_chiusura', \
			FDateTimeReadOnly('data_chiusura','Closed on','Data chiusura','','formtable',True,True) )
		# Details
		self.addDetail("FTodo")
		self.addDetail("FTimetrack")
		# Beautifying
		self.removeFromGroup('name','')
		self.addToGroup('name','todo',0)
	def getDetailIcon(self):
		return "icons/task_16x16.gif"
	def getDetailTitle(self):
		return 'Task'
	def getDetailColumnNames(self):
		ret = FObject.getDetailColumnNames(self)
		ret.remove('description')
		for x in ["priority","data_segnalazione","fk_segnalato_da","fk_cliente","fk_progetto","fk_tipo","stato","descrizione","intervento","data_chiusura"]:
			ret.append(x)
		return ret
	def getDecodeGroupNames(self):
		ret = FObject.getDecodeGroupNames(self)
		ret['']='Details'
		ret['todo']='Task'
		return ret
	def getFilterForm(self):
		return FTodoFilter()
	def getFilterFields(self):
		return ['name','priority','data_segnalazione','fk_segnalato_da','fk_cliente','fk_progetto','father_id','fk_tipo','stato','data_chiusura']
	def getListTitle(self):
		return 'Tasks'
	def getListColumnNames(self):
		return ['priority','data_segnalazione','fk_cliente','fk_progetto','father_id','name','fk_tipo','stato','data_chiusura']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBETodo")()
		try:
			return DBETodo()
		except Exception,e:
			return None
	def getActions(self):
		ret = FObject.getActions(self)
		ret['start_timetracking'] = ['Start Time Tracking','play.png']
		ret['stop_timetracking'] = ['Stop Time Tracking','stop.png']
		return ret
formschema_type_list["FTodo"]=FTodo

class FTodoFilter(FTodo):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FTodo.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'from_data_chiusura', \
			FDateTime('from_data_chiusura','Closed on >=','Data chiusura','','formtable',True,False) )
		self.addField('',-1,'to_data_chiusura', \
			FDateTime('to_data_chiusura','Closed on <=','Data chiusura','','formtable',True,False) )
		self.addField('',-1,'from_stato', \
			FPercent('from_stato','State (%) >=','Stato di avanzamento %','','formtable') )
		self.addField('',-1,'to_stato', \
			FPercent('to_stato','State (%) <=','Stato di avanzamento %','99','formtable') )
		# Details
		self.addDetail("FTodo")
		self.addDetail("FTimetrack")
	def getDetailIcon(self):
		return "icons/task_16x16.gif"
	def getDetailTitle(self):
		return 'Todo Filter'
	def getDetailColumnNames(self):
		return ['name','priority','data_segnalazione','fk_segnalato_da','fk_cliente','fk_progetto','father_id','fk_tipo','stato','descrizione','intervento','data_chiusura','owner','group_id','permissions']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date']
	def getFilterForm(self):
		return FTodoFilter()
	def getFilterFields(self):
		return ['name','priority','data_segnalazione','fk_segnalato_da','fk_cliente','fk_progetto','father_id','fk_tipo','from_stato','to_stato']
	def getListTitle(self):
		return 'Todos'
	def getListColumnNames(self):
		return ['priority','data_segnalazione','fk_cliente','fk_progetto','father_id','name','fk_tipo','stato','data_chiusura']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBETodo")()
		try:
			return DBETodo()
		except Exception,e:
			return None
formschema_type_list["FTodoFilter"]=FTodoFilter

class FTodoTipo(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'order_position', \
			FString('order_position','Order','Ordine',50,255,'','formtable') )
	def getDetailTitle(self):
		return 'Todo::Kind'
	def getDetailColumnNames(self):
		return ['name','description','order_position']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date']
	def getListTitle(self):
		return 'Todo::Kind'
	def getListColumnNames(self):
		return ['order_position','name','description']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBETodoTipo")()
		try:
			return DBETodoTipo()
		except Exception,e:
			return None
formschema_type_list["FTodoTipo"]=FTodoTipo

# #### Projects: end.

class FBanned(FObject):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FObject.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'ban_ip', \
			FString('ban_ip','IP','Banned IP',50,255,'','formtable') )
		self.addField('',-1,'redirected_to', \
			FString('redirected_to','Redirect to','Redirect to...',50,255,'http://adf.ly/XdZw','formtable') )
		self.addField('',-1,'give_reason', \
			FString('give_reason','Given reason','Displayed reason to the banned IP',50,255,'Your IP has been blocked.<br/>See http://adf.ly/XdZw for more details.','formtable') )
	def getDetailIcon(self):
		return "icons/banned_16x16.png"
	def getDetailTitle(self):
		return "Banned"
	def getDetailColumnNames(self):
		return ["creation_date","creator","ban_ip","description","redirected_to","give_reason","owner","group_id","permissions"]
	def getDetailReadOnlyColumnNames(self):
		return ["creator","creation_date","last_modify","last_modify_date"]
	def getFilterForm(self):
		return FBannedFilter()
	def getListTitle(self):
		return "Banlist"
	def getListColumnNames(self):
		return ["creation_date","ban_ip","description","redirected_to","give_reason"]
	def getPagePrefix(self):
		return "dbe"
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEBanned")()
		try:
			return DBEBanned()
		except Exception,e:
			return None
	def getShortDescription(self):
		return "%s: %s" % (self.getValue('ban_ip'),self.getValue('description'))
formschema_type_list["FBanned"]=FBanned

class FBannedFilter(FBanned):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FBanned.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'from_creation_date', \
			FDateTime('from_creation_date','Created >=','Data creazione','','formtable',True,False) )
		self.addField('',-1,'to_creation_date', \
			FDateTime('to_creation_date','Created <','Data creazione','','formtable',True,False) )
	def getDetailIcon(self):
		return "icons/banned_16x16.png"
	def getDetailTitle(self):
		return "Banned Filter"
	def getDetailColumnNames(self):
		return ["creation_date","creator","ban_ip","description","redirected_to","give_reason","owner","group_id","permissions"]
	def getDetailReadOnlyColumnNames(self):
		return ["creator","creation_date","last_modify","last_modify_date"]
	def getFilterForm(self):
		return FBannedFilter()
	def getFilterFields(self):
		return ["from_creation_date","to_creation_date","ban_ip","description"]
	def getListTitle(self):
		return "Banlist"
	def getListColumnNames(self):
		return ["creation_date","ban_ip","description","redirected_to","give_reason"]
	def getPagePrefix(self):
		return "dbe"
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEBanned")()
		try:
			return DBEBanned()
		except Exception,e:
			return None
formschema_type_list["FBannedFilter"]=FBannedFilter

class FMail(FFile):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FFile.__init__(self,nome,azione,metodo,enctype,dbmgr)
		_mydbe=self.getDBE(dbmgr)
		# Fields
		self.addField('email',-1,'msgid', \
			FString('msgid','Message ID','Message ID',50,255,'','formtable') )
		self.addField('email',-1,'msgdate', \
			FDateTime('msgdate','Date','Date',dbmgr.getTodayString(),'formtable',True,True) )
		self.addField('email',-1,'msgfrom', \
			FString('msgfrom','From','From',50,255,'','formtable') )
		self.addField('email',-1,'msgto', \
			FString('msgto','To','To',50,255,'','formtable') )
		self.addField('email',-1,'msgcc', \
			FString('msgcc','CC','CC',50,255,'','formtable') )
		self.addField('email',-1,'subject', \
			FString('subject','Subject','Subject',50,255,'','formtable') )
		self.addField('email',-1,'msgbody', \
			FTextArea('msgbody','Body','Message body',255,'','formtable',-1,-1,True) )
		self.removeFromGroup('filename','file')
		#self.removeFromGroup('description','')
		self.addToGroup('filename','email',0)
		#self.addToGroup('description','file',1)
	def getDetailIcon(self):
		return "icons/email_16x16.png"
	def getDetailTitle(self):
		return 'Email'
	def getDetailColumnNames(self):
		ret = FFile.getDetailColumnNames(self)
		for x in ["msgid","msgdate","msgfrom","msgto","msgcc","subject","msgbody"]:
			ret.append(x)
		return ret
	def getDetailReadOnlyColumnNames(self):
		ret = FFile.getDetailReadOnlyColumnNames(self)
		for x in ["msgid","msgdate","msgfrom","msgto","msgcc","subject","msgbody"]:
			ret.append(x)
		return ret
	def getFilterForm(self):
		return FMailFilter()
	def getFilterFields(self):
		return ['father_id','name','description']
	def getListTitle(self):
		return 'Emails'
	def getGroupNames(self):
		return ['email','file','','_permission']
	def getDecodeGroupNames(self):
		ret = FFile.getDecodeGroupNames(self)
		ret['email']='Email'
		return ret
	def getListColumnNames(self):
		return ['msgdate','msgfrom','subject','msgto','msgcc']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEMail")()
		try:
			return DBEMail()
		except Exception,e:
			return None
formschema_type_list["FMail"]=FMail

class FMailFilter(FMail):
	def __init__(self,nome='',azione='',metodo="POST",enctype='',dbmgr=None):
		FMail.__init__(self,nome,azione,metodo,enctype,dbmgr)
		# Fields
		self.addField('',-1,'from_msgdate', \
			FDateTime('from_msgdate','Date >=','Date','','formtable',True,False) )
		self.addField('',-1,'to_msgdate', \
			FDateTime('to_msgdate','Date <','Date','','formtable',True,False) )
	def getDetailIcon(self):
		return "icons/email_16x16.png"
	def getDetailTitle(self):
		return 'Email Filter'
	def getDetailColumnNames(self):
		return ['father_id','path','filename','checksum','mime','alt_link','owner','group_id','permissions','msgdate','msgfrom','msgto','msgcc','subject','msgbody', 'msgid']
	def getDetailReadOnlyColumnNames(self):
		return ['creator','creation_date','last_modify','last_modify_date','checksum','mime','msgdate','msgfrom','msgto','msgcc','subject','msgbody']
	def getFilterForm(self):
		return FMailFilter()
	def getFilterFields(self):
		return ['from_msgdate','to_msgdate','msgfrom','msgto','msgcc','subject']
	def getListTitle(self):
		return 'Emails'
	def getListColumnNames(self):
		return ['msgdate','msgfrom','msgto','msgcc','subject']
	def getPagePrefix(self):
		return 'dbe'
	def getDBE(self,dbmgr=None):
		if not dbmgr is None:
			return dbmgr.getClazzByTypeName("DBEMail")()
		try:
			return DBEMail()
		except Exception,e:
			return None
# FIXME formschema_type_list["FMailFilter"]=FMailFilter






# -*- coding: utf-8 -*-
#
# @copyright &copy; 2011 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: mydb.py $
# @package dblayer
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

from rprj.dblayer import *

from psycopg2 import *

class PGConnectionProvider(DBConnectionProvider):
	"""PostgreSQL Connection provider"""
	
	__super_init = DBConnectionProvider.__init__
	
	def __init__(self, host, db, user, pwd, verbose=0):
		self.__super_init( host, db, user, pwd, verbose )

	def getConnection(self):
		if self._conn is None:
			try:
				connString = "host=%s dbname=%s user=%s password=%s" % (self._host,self._db,self._user,self._pwd)
				#if self._verbose: print "PGConnectionProvider.getConnection: connString=%s" % ( connString )
				self._conn = connect( connString )
				self._conn.autocommit = True
			except Exception, e:
				if self._verbose: print "PGConnectionProvider.getConnection: Error=%s" % (e)
		return self._conn

	def freeConnection(self,conn):
		try:
			conn.close()
		except Exception, e:
			if self._verbose: print "PGConnectionProvider.freeConnection: Error=%s" % (e)
		del conn
		del self._conn
		self._conn = None

	def getDBType(self):
		return "POSTGRESQL"

	def dbeType2dbType(self,dbetype):
		"""TO OVERRIDE"""
		ret = dbetype
		if dbetype=="uuid" or dbetype==u"uuid":
			ret = "varchar(16)"
		elif dbetype=="datetime":
			ret = "timestamp"
		return ret
	def dbType2dbeType(self,dbetype):
		"""TO OVERRIDE"""
		ret = dbetype
		if dbetype=="varchar(16)":
			ret = "uuid"
		elif dbetype=="timestamp":
			ret = "datetime"
		return ret
	def dbeConstraints2dbConstraints(self,constraints):
		return constraints\
			.replace("not null default '0000-00-00'","default null")\
			.replace("not null default '0000-00-00 00:00:00'","default null")
			#.replace("00:00:00","00:00")




# -*- coding: utf-8 -*-
#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: qtgui.py $
# @package formulator
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""QT Implementation of the formulator"""

import datetime,os,os.path,sys,traceback
import collections # for OrderedDict
from PyQt4 import QtCore,QtGui
try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

import rprj.formulator
from rprj.qtwidgets.editwidgetcontroller import EditWidgetController

class FField(rprj.formulator.FField,QtCore.QObject):
	def __init__(self, aNomeCampo, aTitle, aDescription, aSize, aLength=20, aValore='', aClasseCss=None, myform=None, mytipo='s'):
		QtCore.QObject.__init__(self)
		rprj.formulator.FField.__init__(self, aNomeCampo, aTitle, aDescription, aSize, aLength=20, aValore='', aClasseCss=None, myform=None, mytipo='s')
		self.widget = None
		self.label = None
		self.field = None
	def render(self,parent):
		if self.widget is None:
			self.InitWidget(parent)
		if not self.widget is None:
			self.widget.setVisible(True)
		if not self.field is None:
			self.field.setEnabled(True)
			self.field.setVisible(True)
		return self.widget
	def render_view(self,parent):
		return self.render_readonly(parent)
	def render_hidden(self,parent):
		self.render(parent)
		if not self.widget is None:
			self.widget.setVisible(False)
		elif not self.field is None:
			self.field.setVisible(False)
		return self.widget
	def render_readonly(self,parent):
		if self.widget is None:
			self.InitWidget(parent)
		self.field.setEnabled(False)
		return self.widget
	def InitLabel(self,parent):
		self.label = QtGui.QLabel(parent)
		self.label.setText(QtGui.QApplication.translate("Formulator", self._title, None, QtGui.QApplication.UnicodeUTF8))
		self.label.setObjectName("label_%s" % self.aNomeCampo)
		return self.label
	def InitField(self,parent):
		self.field = QtGui.QLineEdit(parent)
		# Size Policy: fix expanding issue on mac
		sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
		sizePolicy.setHorizontalStretch(1)
		sizePolicy.setVerticalStretch(0)
		self.field.setSizePolicy( sizePolicy )
		self.field.setObjectName("field_%s" % self.aNomeCampo)
		return self.field
	def InitWidget(self,parent):
		if parent is None:
			return
		self.widget = QtGui.QWidget(parent)
		self.widget.setObjectName("widget_%s"%self.aNomeCampo)
		self.layout = QtGui.QVBoxLayout(self.widget)
		self.layout.setMargin(0)
		self.layout.setObjectName("layout_%s"%self.aNomeCampo)
		self.InitLabel(self.widget)
		self.layout.addWidget(self.label)
		self.InitField(self.widget)
		self.layout.addWidget(self.field)
	def getValue(self):
		if not self.field is None:
			self.aValore = "%s" % self.field.text()
		return rprj.formulator.FField.getValue(self)
	def setValue(self,v, update_widget=True):
		rprj.formulator.FField.setValue(self,v)
		if not self.field is None:
			self.field.setText("%s"%v)

class FForm(rprj.formulator.FForm,QtCore.QObject):
	def __init__(self, nome='', azione='', metodo="POST", enctype='',dbmgr=None):
		QtCore.QObject.__init__(self)
		rprj.formulator.FForm.__init__(self, nome, azione, metodo, enctype,dbmgr)
		self.widget=None
		self.filterWidget=None
		self.buttons={}
	def render(self,parent):
		if self.widget is None:
			self.InitWidget(parent)
		if not self.widget is None:
			self.widget.setVisible(True)
		return self.widget
	def render_filter(self,parent):
		if self.filterWidget is None:
			self.InitFilterWidget(parent)
		return self.filterWidget
	# Search Filter: start.
	def InitFilterWidget(self,parent):
		if parent is None:
			return
		self.filterWidget = QtGui.QWidget(parent)
		self.filterWidget.setObjectName("widget_%s"%self.nome)
		self.filterLayout = QtGui.QVBoxLayout(self.filterWidget)
		self.filterLayout.setMargin(0)
		self.filterLayout.setObjectName("layout_%s"%self.nome)
		# Actions
		self.filterActions = QtGui.QWidget(self.widget)
		self.filterActions.setObjectName("widget_%s"%self.nome)
		self.filterLayoutActions = QtGui.QHBoxLayout(self.filterActions)
		self.filterLayoutActions.setMargin(0)
		self.filterLayoutActions.setObjectName("layout_%s"%self.nome)
		self.filterLayout.addWidget(self.filterActions)
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(":/rPrj/icons/glass.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		buttonShowHide = QtGui.QPushButton(icon,"",self.filterActions)
		buttonShowHide.setToolTip("Show / hide search filters")
		QtCore.QObject.connect(buttonShowHide,QtCore.SIGNAL("clicked()"),self.slotShowHideFilter)
		self.filterLayoutActions.addWidget(buttonShowHide)
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(":/rPrj/icons/reload.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		button = QtGui.QPushButton(icon,"",self.filterActions)
		button.setToolTip("Search")
		QtCore.QObject.connect(button,QtCore.SIGNAL("clicked()"),self.slotSearch)
		self.filterLayoutActions.addWidget(button)
		# Search Fields
		self.filterFields = QtGui.QWidget(self.filterWidget)
		formLayout = QtGui.QFormLayout(self.filterFields)
		formLayout.setFieldGrowthPolicy(QtGui.QFormLayout.ExpandingFieldsGrow)
		filters = self.getFilterFields()
		for myfieldname in filters:
			myfield = self.getField(myfieldname)
			label = myfield.InitLabel(self.filterFields)
			field = None
			if isinstance(myfield,FKField):
				field = myfield.InitField(self.filterFields,self.dbmgr,self.formFactory)
			else:
				field = myfield.InitField(self.filterFields)
			formLayout.addRow(label, field)
		self.filterFields.setVisible( False )
		self.filterLayout.addWidget(self.filterFields)
		if len(filters)==0:
			buttonShowHide.setVisible(False)
	def slotShowHideFilter(self):
		self.filterFields.setVisible( not self.filterFields.isVisible() )
	def slotSearch(self):
		self.filterWidget.emit(QtCore.SIGNAL("doSearch()"))
	# Search Filter: end.
	def InitWidget(self,parent):
		if parent is None:
			return
		self.widget = QtGui.QWidget(parent)
		self.widget.setObjectName("widget_%s"%self.nome)
		self.layout = QtGui.QVBoxLayout(self.widget)
		self.layout.setMargin(0)
		self.layout.setObjectName("layout_%s"%self.nome)
		# Actions
		self.widgetActions = QtGui.QWidget(self.widget)
		self.widgetActions.setObjectName("widget_%s"%self.nome)
		self.layoutActions = QtGui.QHBoxLayout(self.widgetActions)
		self.layoutActions.setMargin(0)
		self.layoutActions.setObjectName("layout_%s"%self.nome)
		myactions = collections.OrderedDict( (\
			('close', ['Close', '_do.php', 'icons/fileclose.png', 'Close']),\
			('reload', ['Reload', '_do.php', 'icons/reload.png', 'Reload']),\
			('save', ['Save', '_do.php', 'icons/filesave.png', 'Save']),\
			('delete', ['Delete', '_do.php', 'icons/editdelete.png', 'Delete']),\
			) )
		for k,v in self.getActions().items():
			myactions[k]=v
		for k in myactions.keys():
			if len(myactions[k])>2:
				icon = QtGui.QIcon()
				icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myactions[k][2])), QtGui.QIcon.Normal, QtGui.QIcon.Off)
				button = QtGui.QPushButton(icon,"",self.widgetActions)
				button.setToolTip(myactions[k][0])
			else:
				button = QtGui.QPushButton(myactions[k][0],self.widgetActions)
			self.buttons[k]=button
			button.setObjectName( "button_%s_%s" % (self.nome,k) )
			tmpmsg = "button_%s_%s" % (self.nome,k)
			myaction = lambda:self.emit(QtCore.SIGNAL("clickedButton(PyQt_PyObject)"),"%s"%button.sender().objectName())
			QtCore.QObject.connect(button,QtCore.SIGNAL("clicked()"),myaction)
			self.layoutActions.addWidget(button,0)
		self.layoutActions.addItem(QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum))
		self.layout.addWidget(self.widgetActions,0)
		# Tab
		self.widgetTab = QtGui.QTabWidget(self.widget)
		self.widgetTab.setObjectName("widget_%s_tab"%self.nome)
		self.layoutTab = QtGui.QVBoxLayout(self.widgetTab)
		self.layoutTab.setMargin(0)
		self.layoutTab.setObjectName("layout_%s_tab"%self.nome)
		for groupName in self.getGroupNames():
			groupBox = QtGui.QWidget(self.widgetTab)
			decodedGroupName = self.decodeGroupName(groupName)
			if decodedGroupName==groupName:
				decodedGroupName = self.getDetailTitle()
			groupBox.setObjectName("groupBox_%s"%groupName)
			formLayout = QtGui.QFormLayout(groupBox)
			# Field Growth Policy: fix expanding issue on mac
			formLayout.setFieldGrowthPolicy(QtGui.QFormLayout.ExpandingFieldsGrow)
			formLayout.setObjectName("gridLayout_%s"%groupName)
			mygroup = self.getGroup(groupName)
			for myfieldname in mygroup:
				myfield = self.getField(myfieldname)
				label = myfield.InitLabel(groupBox)
				field = None
				if isinstance(myfield,FKField):
					field = myfield.InitField(groupBox,self.dbmgr,self.formFactory)
				else:
					field = myfield.InitField(groupBox)
				if myfieldname in self.getDetailReadOnlyColumnNames():
					field.setEnabled(False)
				if not myfieldname in self.getDetailColumnNames():
					label.setVisible(False)
					field.setVisible(False)
				else:
					formLayout.addRow(label, field)
			self.widgetTab.addTab(groupBox,decodedGroupName)
		self.layout.addWidget(self.widgetTab,1)
	def setWidget(self,w):
		if w==self.widget:
			return
		self.widget=w
		for fieldName in self.getFieldNames():
			childs = w.findChildren(QtGui.QWidget,"field_%s" % fieldName)
			if len(childs)==1:
				myField = rprj.formulator.FMasterDetail.getField( self, fieldName )
				myField.field = childs[0]
				myField.render(None)
	def setValues( self, valori ):
		rprj.formulator.FForm.setValues( self, valori )

class FMasterDetail(FForm,rprj.formulator.FMasterDetail):
	def __init__(self,nome='', azione='', metodo="POST",enctype='',dbmgr=None):
		rprj.formulator.FMasterDetail.__init__(self,nome, azione, metodo,enctype,dbmgr)
		FForm.__init__(self,nome,azione,metodo,enctype,dbmgr)
	def getValues(self):
		tmp = rprj.formulator.FMasterDetail.getValues(self)
		# Because of setWidget, I have non-existent widgets and the getValue returns None :-(
		# this creates problems in xmlrpc marshalling :-P
		ret = {}
		for k in tmp.keys():
			v = tmp[k]
			if v is None:
				continue
			ret[k] = v
		return ret
	def setValues( self, valori, doFetchChilds=False ):
		rprj.formulator.FMasterDetail.setValues( self, valori, doFetchChilds )

FAssociation=rprj.formulator.FAssociation

FormFactory=rprj.formulator.FormFactory

# ######################### Campi Standard

class FFileField(FField,rprj.formulator.FFileField):
	def __init__(self,aNomeCampo,aTitle,aDescription,dest_directory='',aSize=1,aLength=20,aValore='',aClasseCss=None):
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
		rprj.formulator.FFileField.__init__(self,aNomeCampo,aTitle,aDescription,dest_directory,aSize,aLength,aValore,aClasseCss)
	def InitField(self,parent):
		self.fieldWidget = QtGui.QWidget(parent)
		self.fileDialog = QtGui.QFileDialog(parent,QtGui.QApplication.translate("Formulator", "Select file...", None, QtGui.QApplication.UnicodeUTF8),os.path.expanduser("~"))
		self.fileDialog.setFileMode(QtGui.QFileDialog.ExistingFile)
		self.layout = QtGui.QHBoxLayout(self.fieldWidget)
		self.layout.setMargin(0)
		self.layout.setObjectName("layout_%s"%self.aNomeCampo)
		# TODO widget for the thumbnail
		myfield = FField.InitField(self,self.fieldWidget)
		myfield.setEnabled(False)
		self.layout.addWidget(myfield,1)
		self.icon = QtGui.QIcon()
		self.icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/folder_16x16.gif")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.mybutton = QtGui.QPushButton(self.icon,"",self.fieldWidget)
		self.layout.addWidget(self.mybutton,0)
		QtCore.QObject.connect(self.mybutton,QtCore.SIGNAL("clicked()"),self.slotFileDialog)
		return self.fieldWidget
	def slotFileDialog(self):
		fileNames = []
		if self.fileDialog.exec_():
			fileNames = self.fileDialog.selectedFiles()
		if len(fileNames)!=1:
			return
		self.setValue( "%s" % fileNames[0] )
	def setValue(self,v):
		FField.setValue(self,v)
		if self.myform.isImage():
			# TODO isImage=True
			print "FFileField.setValue: TODO - isImage=True"
class FNumber(FField): #,rprj.formulator.FNumber):
	def getValue(self):
		ret = FField.getValue(self)
		if ret=='':
			return None
		return ret
class FPercent(FField): #,rprj.formulator.FPercent):
	def getValue(self):
		ret = FField.getValue(self)
		if ret=='': return None
		return ret
class FString(FField,rprj.formulator.FString):
	def getValue(self):
		if not self.field is None:
			self.aValore = "%s" % self.field.text()
		return rprj.formulator.FField.getValue(self)
class FPassword(FString): #,rprj.formulator.FPassword):
	def InitField(self,parent):
		FString.InitField(self,parent)
		# TEST this
		self.field.setEchoMode(QtGui.QLineEdit.Password)
		return self.field
class FServerFile(FField): #,rprj.formulator.FServerFile):
	pass
class FServerImage(FField): #,rprj.formulator.FServerImage):
	pass
class FList(FField,rprj.formulator.FList):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength=20,aValore='',aClasseCss=None,listaValori={},altezza=1,multiselezione=False ):
		"""listaValori = {  k=>v  }
		altezza = numero di elementi visualizzati della select
		multiselezione"""
		rprj.formulator.FList.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss,listaValori,altezza,multiselezione )
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aLength,aValore,aClasseCss)
	def InitField(self,parent):
		# TODO multiselection and altezza (height)
		self.field = QtGui.QComboBox(parent)
		self.field.setObjectName("field_%s" % self.aNomeCampo)
		for k in self.listaValori.keys():
			self.field.addItem(self.listaValori[k],k)
		self.field.setEditable(False)
		return self.field
	def getValue(self):
		if not self.field is None:
			self.aValore = self.field.itemData( self.field.currentIndex() ).toPyObject()
		return rprj.formulator.FField.getValue(self)
	def setValue(self,v, update_widget=True):
		rprj.formulator.FField.setValue(self,v)
		if not self.field is None:
			self.field.setCurrentIndex( self.field.findData(v) )
class FCheckBox(FField): #,rprj.formulator.FCheckBox):
	pass
class FTextArea(FField,rprj.formulator.FTextArea):
	def __init__(self,aNomeCampo,aTitle,aDescription,aSize,aValore='',aClasseCss=None,width=None,height=None,basicFormatting=True):
		rprj.formulator.FTextArea.__init__(self,aNomeCampo,aTitle,aDescription,aSize,aValore,aClasseCss,width,height,basicFormatting)
		FField.__init__(self,aNomeCampo,aTitle,aDescription,aSize,-1,aValore)
	def InitField(self,parent):
		self.field = QtGui.QTextEdit(parent)
		self.field.setObjectName("field_%s" % self.aNomeCampo)
		hp = QtGui.QSizePolicy.Expanding
		vp = QtGui.QSizePolicy.Expanding
		if self.width>0:
			hp = QtGui.QSizePolicy.Fixed
			self.field.setMaximumWidth(self.width*14)
		if self.height>0:
			vp = QtGui.QSizePolicy.Fixed
			self.field.setMaximumHeight(self.height*22)
		sizePolicy = QtGui.QSizePolicy(hp, vp)
		sizePolicy.setHorizontalStretch( self.width<=0 )
		sizePolicy.setVerticalStretch( self.height<=0 )
		self.field.setSizePolicy(sizePolicy)
		return self.field
	def getValue(self):
		if not self.field is None:
			self.aValore = "%s" % self.field.toPlainText()
		return rprj.formulator.FField.getValue(self)
class FDateTime(FField,rprj.formulator.FDateTime):
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore,aClasseCss=None,aVisualizzaData=True,aVisualizzaOra=True):
		rprj.formulator.FDateTime.__init__(self,aNomeCampo,aTitle,aDescription,aValore,aClasseCss,aVisualizzaData,aVisualizzaOra)
		FField.__init__(self,aNomeCampo,aTitle,aDescription,-1,-1,aValore)
	def InitField(self,parent):
		if self.aVisualizzaData:
			if self.aVisualizzaOra:
				self.field = QtGui.QDateTimeEdit(parent)
			else:
				self.field = QtGui.QDateEdit(parent)
		else:
			self.field = QtGui.QTimeEdit(parent)
		self.field.setObjectName("field_%s" % self.aNomeCampo)
		# Forced rendering with default value
		self.setValue(self.aValore)
		return self.field
	def setValue(self,v, update_widget=True):
		rprj.formulator.FField.setValue(self,v)
		if not self.field is None:
			if v is None:
				self.field.setDateTime(QtCore.QDateTime.fromString('2000-01-01 00:00:00',"yyyy-MM-dd hh:mm:ss"))
			elif type(v)==str or type(v)==unicode:
				self.field.setDateTime(QtCore.QDateTime.fromString(v,"yyyy-MM-dd hh:mm:ss"))
			elif isinstance(v,datetime.time):
				# TODO show only time for timetracks
				self.field.setTime(QtCore.QTime(v.hour,v.minute,v.second))
			elif isinstance(v,datetime.datetime):
				self.field.setDateTime(QtCore.QDateTime(v.year,v.month,v.day,v.hour,v.minute,v.second))
			else:
				self.field.setDateTime(QtCore.QDateTime(v))
	def getValue(self):
		if not self.field is None:
			self.aValore = self.field.dateTime().toPyDateTime()
			if '2000-01-01 00:00:00'=="%s"%self.aValore:
				self.aValore='0000-00-00 00:00:00'
		return rprj.formulator.FField.getValue(self)
	def clean(self):
		"""Resets the field value"""
		self.setValue(None)
class FDateTimeReadOnly(FDateTime): #,rprj.formulator.FDateTimeReadOnly):
	def InitField(self,parent):
		FDateTime.InitField(self,parent)
		self.field.setEnabled(False)
		return self.field
	def render(self,parent):
		return self.render_readonly(parent)
class FKField(FField,rprj.formulator.FKField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore=None,aClasseCss=None,mydbe=None,myFK=None,description_columns=[],destform=None,viewmode='select',aDescription_glue=" - ",aAltezza=1,aMultiselezione=False):
		rprj.formulator.FKField.__init__(self,aNomeCampo,aTitle,aDescription,aValore,aClasseCss,mydbe,myFK,description_columns,destform,viewmode,aDescription_glue,aAltezza,aMultiselezione)
		FField.__init__(self,aNomeCampo,aTitle,aDescription,0,0,aValore,aClasseCss)
		self.dbmgr = None
		self.formFactory = None
		self._mythread = QtCore.QThread()
		self._mythread.run = self.runDecodeValue
	def InitField(self,parent,dbmgr,formFactory):
		self.fieldWidget = QtGui.QWidget(parent)
		self.layout = QtGui.QHBoxLayout(self.fieldWidget)
		self.layout.setMargin(0)
		self.layout.setObjectName("layout_%s"%self.aNomeCampo)
		# TODO widget for the thumbnail
		myfield = FField.InitField(self,self.fieldWidget)
		myfield.setEnabled(False)
		self.layout.addWidget(myfield,1)
		self.icon = QtGui.QIcon()
		self.icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/glass.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.mybutton = QtGui.QPushButton(self.icon,"",self.fieldWidget)
		self.layout.addWidget(self.mybutton,0)
		QtCore.QObject.connect(self.mybutton,QtCore.SIGNAL("clicked()"),self.slotSearch)
		self.dbmgr=dbmgr
		self.formFactory=formFactory
		return self.fieldWidget
	def getMyFilterForm(self):
		myclazz = self.dbmgr.getClazz(self.myFK.tabella_riferita)
		return self.formFactory.getInstanceByDBEName( myclazz().getTypeName(),dbmgr=self.dbmgr )
	def slotSearch(self):
		myfilterform = self.getMyFilterForm()
		searchDialog = QtGui.QDialog(self.fieldWidget)
		searchDialog.setModal(True)
		searchDialog.setWindowTitle(QtGui.QApplication.translate("FKFiels", "Select", None, QtGui.QApplication.UnicodeUTF8))
		verticalLayout = QtGui.QVBoxLayout(searchDialog)
		verticalLayout.setMargin(0)
		verticalLayout.setObjectName("%s_verticalLayout"%self.aNomeCampo)
		import searchwidget
		searchWidget = searchwidget.SearchWidget(searchDialog,"%s_explorer"%self.aNomeCampo,myfilterform,True)
		searchWidget.setServer(self.dbmgr)
		searchWidget.setFormFactory(self.formFactory)
		if not self.getValue() is None and not myfilterform.getDetailTitle()=='Object':
			selectedDBE = myfilterform.getDBE(self.dbmgr)
			selectedDBE.setValue(self.myFK.colonna_riferita,self.getValue())
			searchWidget.setSelectedDBE(selectedDBE)
			myfilterform.setValue(self.myFK.colonna_riferita,self.getValue())
			searchWidget.slotDoSearch()
			myfilterform.setValue(self.myFK.colonna_riferita,None)
		QtCore.QObject.connect(searchWidget,QtCore.SIGNAL("selected(QListWidgetItem,PyQt_PyObject)"),searchDialog.accept)
		verticalLayout.addWidget(searchWidget,1)
		res = searchDialog.exec_()
		if res==0: return
		selectedDBE = searchWidget.selectedDBE()
		self.setValue( selectedDBE.getValue( self.myFK.colonna_riferita ) )
	def clean(self):
		"""Resets the field value"""
		self.setValue(None)
	def getValue(self):
		return rprj.formulator.FField.getValue(self)
	def setValue(self, v, update_widget=True):
		rprj.formulator.FField.setValue(self,v)
		if not self.field is None:
			if v is None:
				self.field.setText( '' )
				return
			# BUG FIXED sqlite and mysql do not supports multithread
			if self.dbmgr.getConnectionProvider().getDBType() in ['MYSQL','SQLite']:
				# If the connection provider does NOT support multithread...
				self.runDecodeValue()
			else:
				# ...otherwise start decode thread
				self.startDecodeValue()
	def startDecodeValue(self):
		if not self._mythread.isRunning():
			self._mythread.start()
	def runDecodeValue(self):
		try:
			v = rprj.formulator.FField.getValue(self)
			if v is None:
				return
			cerca = self.dbmgr.getDBEFactory().getClazz( self.myFK.tabella_riferita )()
			cerca.setValue(self.myFK.colonna_riferita,v)
			lista = self.dbmgr.search(cerca,uselike=False)
			if len(lista)==1:
				mydbe = lista[0]
				description_array = []
				for chiave in self.description_columns:
					description_array.append( mydbe.getValue(chiave) )
				link_desc = self.description_glue.join( description_array )
				self.field.setText( link_desc )
			else:
				pass
		except Exception,e:
			print "FKField.runDecodeValue: ERROR '%s'" % e
			print "".join(traceback.format_tb(sys.exc_info()[2]))
			pass

class FKObjectField(FKField): #,rprj.formulator.FKObjectField):
	def __init__(self,aNomeCampo,aTitle,aDescription,aValore=None,aClasseCss=None,\
					mydbe=None,myFK=None,description_columns=[],destform=None,viewmode='select',aDescription_glue=" - ",aAltezza=1,aMultiselezione=False):
		FKField.__init__(self,aNomeCampo,aTitle,aDescription,aValore,aClasseCss,\
					mydbe,myFK,description_columns,destform,viewmode,aDescription_glue,aAltezza,aMultiselezione)
	def getMyFilterForm(self):
		return self.formFactory.getInstanceByDBEName( "DBEObject",dbmgr=self.dbmgr )
	def setValue(self,v, update_widget=True):
		# TODO use multithreading like FKField
		rprj.formulator.FField.setValue(self,v)
		if not self.field is None:
			myobj = self.dbmgr.objectById("%s"%v)
			if not myobj is None:
				self.field.setText( myobj.getValue('name') )
class FHtml(FField): #,rprj.formulator.FHtml):
	def getValue(self):
		if not self.field is None:
			self.aValore = "%s" % self.field.text()
		return rprj.formulator.FField.getValue(self)
	def InitField(self,parent):
		self.field = EditWidgetController(parent,"field_%s" % self.aNomeCampo)
		sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
		sizePolicy.setHorizontalStretch(0)
		sizePolicy.setVerticalStretch(1)
		sizePolicy.setHeightForWidth(self.field.sizePolicy().hasHeightForWidth())
		self.field.setSizePolicy(sizePolicy)
		return self.field
class FUuid(FString,rprj.formulator.FUuid):
	def InitField(self,parent):
		FString.InitField(self,parent)
		self.field.setEnabled(False)
		return self.field
	def getValue(self):
		return rprj.formulator.FField.getValue(self)
	def setValue(self,v, update_widget=True):
		rprj.formulator.FField.setValue(self,v)
		_v = ("%s" % v).replace("uuid","")
		__v = "%s-%s-%s-%s-%s" % (_v[:8],_v[8:12],_v[12:16],_v[16:20],_v[20:])
		if not self.field is None:
			self.field.setText(__v)
class FPermissions(FField): #,rprj.formulator.FPermissions):
	def __init__(self, aNomeCampo, aTitle, aDescription, aSize, aLength=20, aValore='', aClasseCss=None, myform=None, mytipo='s'):
		QtCore.QObject.__init__(self)
		rprj.formulator.FField.__init__(self, aNomeCampo, aTitle, aDescription, aSize, aLength=20, aValore='', aClasseCss=None, myform=None, mytipo='s')
		self.widget = None
		self.label = None
		self.field = None
		self.buttonUserRead = None
		self.buttonUserWrite = None
		self.buttonUserExecute = None
		self.buttonGroupRead = None
		self.buttonGroupWrite = None
		self.buttonGroupExecute = None
		self.buttonAllRead = None
		self.buttonAllWrite = None
		self.buttonAllExecute = None
	def getValue(self):
		if self.buttonUserRead is None:
			# Field is not visible, then return what we got
			return self.aValore
		v=[ '-', '-', '-', '-', '-', '-', '-', '-', '-' ]
		if self.buttonUserRead.isChecked(): v[0]='r'
		if self.buttonUserWrite.isChecked(): v[1]='w'
		if self.buttonUserExecute.isChecked(): v[2]='x'
		if self.buttonGroupRead.isChecked(): v[3]='r'
		if self.buttonGroupWrite.isChecked(): v[4]='w'
		if self.buttonGroupExecute.isChecked(): v[5]='x'
		if self.buttonAllRead.isChecked(): v[6]='r'
		if self.buttonAllWrite.isChecked(): v[7]='w'
		if self.buttonAllExecute.isChecked(): v[8]='x'
		self.aValore = "".join(v)
		return self.aValore
	def setValue(self,v, update_widget=True):
		if len(v)<9:
			v = "%s%s" % (v, '-' * (9-len(v)) )
		self.aValore = v
		if self.buttonUserRead is None:
			return
		self.buttonUserRead.setChecked( v[0]=='r' )
		self.buttonUserWrite.setChecked( v[1]=='w' )
		self.buttonUserExecute.setChecked( v[2]=='x' )
		self.buttonGroupRead.setChecked( v[3]=='r' )
		self.buttonGroupWrite.setChecked( v[4]=='w' )
		self.buttonGroupExecute.setChecked( v[5]=='x' )
		self.buttonAllRead.setChecked( v[6]=='r' )
		self.buttonAllWrite.setChecked( v[7]=='w' )
		self.buttonAllExecute.setChecked( v[8]=='x' )
	def InitField(self,parent):
		self.field = QtGui.QWidget(parent)
		self.field.setObjectName("field_%s"%self.aNomeCampo)
		self.fieldLayout = QtGui.QGridLayout(self.field)
		self.fieldLayout.setMargin(0)
		self.fieldLayout.setObjectName("field_layout_%s"%self.aNomeCampo)
		# Buttons
		col = 0
		self.userLabel = QtGui.QLabel("User", self.field)
		self.fieldLayout.addWidget(self.userLabel, 0, col+0,  1, 3, QtCore.Qt.AlignHCenter)
		self.buttonUserRead = QtGui.QToolButton(self.field)
		self.buttonUserRead.setText("R")
		self.buttonUserRead.setToolTip("Read")
		self.buttonUserRead.setCheckable(True)
		self.fieldLayout.addWidget(self.buttonUserRead, 1, col+0)
		self.buttonUserWrite = QtGui.QToolButton( self.field)
		self.buttonUserWrite.setText("W")
		self.buttonUserWrite.setCheckable(True)
		self.buttonUserWrite.setToolTip("Write")
		self.fieldLayout.addWidget(self.buttonUserWrite, 1, col+1)
		self.buttonUserExecute = QtGui.QToolButton( self.field)
		self.buttonUserExecute.setText("X")
		self.buttonUserExecute.setCheckable(True)
		self.buttonUserExecute.setToolTip("Execute")
		self.fieldLayout.addWidget(self.buttonUserExecute, 1, col+2)
		col += 3
		self.groupLabel = QtGui.QLabel("Group", self.field)
		self.fieldLayout.addWidget(self.groupLabel, 0, col+0,  1, 3, QtCore.Qt.AlignHCenter)
		self.buttonGroupRead = QtGui.QToolButton(self.field)
		self.buttonGroupRead.setText("R")
		self.buttonGroupRead.setToolTip("Read")
		self.buttonGroupRead.setCheckable(True)
		self.fieldLayout.addWidget(self.buttonGroupRead, 1, col+0)
		self.buttonGroupWrite = QtGui.QToolButton(self.field)
		self.buttonGroupWrite.setText("W")
		self.buttonGroupWrite.setCheckable(True)
		self.buttonGroupWrite.setToolTip("Write")
		self.fieldLayout.addWidget(self.buttonGroupWrite, 1, col+1)
		self.buttonGroupExecute = QtGui.QToolButton(self.field)
		self.buttonGroupExecute.setText("X")
		self.buttonGroupExecute.setCheckable(True)
		self.buttonGroupExecute.setToolTip("Execute")
		self.fieldLayout.addWidget(self.buttonGroupExecute, 1, col+2)
		col += 3
		self.allLabel = QtGui.QLabel("All", self.field)
		self.fieldLayout.addWidget(self.allLabel, 0, col+0,  1, 3, QtCore.Qt.AlignHCenter)
		self.buttonAllRead = QtGui.QToolButton(self.field)
		self.buttonAllRead.setText("R")
		self.buttonAllRead.setToolTip("Read")
		self.buttonAllRead.setCheckable(True)
		self.fieldLayout.addWidget(self.buttonAllRead, 1, col+0)
		self.buttonAllWrite = QtGui.QToolButton(self.field)
		self.buttonAllWrite.setText("W")
		self.buttonAllWrite.setCheckable(True)
		self.buttonAllWrite.setToolTip("Write")
		self.fieldLayout.addWidget(self.buttonAllWrite, 1, col+1)
		self.buttonAllExecute = QtGui.QToolButton(self.field)
		self.buttonAllExecute.setText("X")
		self.buttonAllExecute.setCheckable(True)
		self.buttonAllExecute.setToolTip("Execute")
		self.fieldLayout.addWidget(self.buttonAllExecute, 1, col+2)
		return self.field
class FLanguage(FField): #,rprj.formulator.FLanguage):
	pass
class FChildSort(FList): #,rprj.formulator.FChildSort):
	# TODO
	def getValue(self):
		return rprj.formulator.FField.getValue(self)
	def setValue(self,v, update_widget=True):
		rprj.formulator.FField.setValue(self,v)
class FIPAddress(FField): #,rprj.formulator.FIPAddress):
	pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""
"""

from ConfigParser import ConfigParser
import os,sys,traceback
from PyQt4 import QtCore, QtGui, Qt
try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s


from rprj.dblayer import DBMgr,createConnection
from rprj.formulator import FormFactory, apps_rc
from rprj.apps import mainwindow,  logindialog, dbschema, formschema

RPRJ_CONFIG = "%s%s.config%srprj%srprj.cfg" % ( os.path.expanduser("~"), os.path.sep, os.path.sep, os.path.sep )
RPRJ_WINDOW_TITLE = ":: R-Prj ::"
RPRJ_LOCAL_DB = "%s%s.config%srprj%srprj.db" % ( os.path.expanduser("~"), os.path.sep, os.path.sep, os.path.sep )
RPRJ_PLUGINS = "%s%s.config%srprj%splugins" % ( os.path.expanduser("~"), os.path.sep, os.path.sep, os.path.sep )

class RApp(QtCore.QObject):
    def __init__(self):
        QtCore.QObject.__init__(self)
        self.LoadConfig(RPRJ_CONFIG)
        self.SetupPlugins(RPRJ_PLUGINS)
        self.localpath = os.path.realpath(os.path.dirname(__file__))
        # Extra Components (web browser, email client, rss reader, ecc.)
        self.components = {}
        # UI
        self.app = QtGui.QApplication([])
        self.app.setApplicationName(RPRJ_WINDOW_TITLE)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/rprj.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.app.setWindowIcon(icon)
        self.loadStyle()
        self.myMainWindow = None
        if self.getPref("UI","Frameless",False):
            self.myMainWindow = QtGui.QMainWindow(None,QtCore.Qt.FramelessWindowHint)
            self.myMainWindow.setAttribute(QtCore.Qt.WA_TranslucentBackground,True)
        else:
            self.myMainWindow = QtGui.QMainWindow()
        self.ui=mainwindow.Ui_MainWindow()
        self.ui.setupUi(self.myMainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/rprj.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.myMainWindow.setWindowIcon(icon)
        # Main Window Setup
        self.progressBar = QtGui.QProgressBar(self.myMainWindow.statusBar())
        self.progressBar.setMaximumWidth(100)
        self.progressBar.setMaximumHeight(20)
        self.myMainWindow.statusBar().addPermanentWidget(self.progressBar,1)
        tmpMainWindowSize = self.getPref("UI","mainWindowSize",[])
        if len( tmpMainWindowSize )==2:
            self.myMainWindow.resize(tmpMainWindowSize[0],tmpMainWindowSize[1])
        else:
            self.myMainWindow.resize(800,800)
        # Login Dialog
        self.loginDialog = QtGui.QDialog(self.myMainWindow)
        self.uiLogin=logindialog.Ui_LoginDialog()
        self.uiLogin.setupUi(self.loginDialog)
        self._loginShowOptions(False)
        # Logic
        self.server = None
        self.dbeFactory = dbschema.dbeFactory
        # Components
        componentmodules = self.getPref("Main","Components",["rprjcomponent"])
        for module in componentmodules:
            try:
                exec("from rprj.apps import %s" % module)
            except:
                exec("import %s" % module)
            clazzname = None
            exec("""
for x in dir(%s):
    if x.find('Component')>0:
        clazzname=x
        break
"""%module)
            exec( "self.addComponent( %s.%s(self.config,self) )" % (module, clazzname) )
        if self.getPref("UI","Frameless","False"):
            self.mybtn_minimize = QtGui.QPushButton(self.ui.menubar)
            self.mybtn_minimize.setText("_")
            self.mybtn_minimize.setObjectName("btn_minimize")
            self.mybtn_maximize = QtGui.QPushButton(self.ui.menubar)
            self.mybtn_maximize.setText("O")
            self.mybtn_maximize.setObjectName("btn_maximize")
            self.mybtn_close = QtGui.QPushButton(self.ui.menubar)
            self.mybtn_close.setText("X")
            self.mybtn_close.setObjectName("btn_close")
            # FIXME this is horrible!!!
            self.mybtn_minimize.move(50,0)
            self.mybtn_maximize.move(30,0)
            self.mybtn_close.move(10,0)
        # Binds
        self.bind()
    def bind(self):
        # Menu
        # Main Menu
        QtCore.QObject.connect(self.ui.actionLogin,QtCore.SIGNAL("triggered()"),self.slotLogin) # 2012.06.16
        QtCore.QObject.connect(self.ui.actionReload_Style,QtCore.SIGNAL("triggered()"),self.slotReloadStylesheet)
        QtCore.QObject.connect(self.ui.actionQuit,QtCore.SIGNAL("triggered()"),self.slotQuit)
        # Login Dialog
        QtCore.QObject.connect(self.uiLogin.buttonBox,QtCore.SIGNAL(_fromUtf8("accepted()")),self.slotLoginOk)
        QtCore.QObject.connect(self.uiLogin.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), self.slotLoginCancel)
        QtCore.QObject.connect(self.uiLogin.pushButton, QtCore.SIGNAL(_fromUtf8("clicked()")), self.slotLoginMoreOptions)
        # Frameless
        if self.getPref("UI","Frameless","False"):
            QtCore.QObject.connect(self.mybtn_minimize,QtCore.SIGNAL("clicked()"),self.slotMinimizeWindow)
            QtCore.QObject.connect(self.mybtn_maximize,QtCore.SIGNAL("clicked()"),self.slotMaximizeWindow)
            QtCore.QObject.connect(self.mybtn_close,QtCore.SIGNAL("clicked()"),self.slotQuit)
    def _loginShowOptions(self,b):
        self.uiLogin.label_3.setVisible(b)
        self.uiLogin.label_4.setVisible(b)
        self.uiLogin.lineEdit_schema.setVisible(b)
        self.uiLogin.comboBox.setVisible(b)
    def loadStyle(self):
        try:
            style = self.getPref("UI","Style","")
            if len(style)==0:
                return
            ss = file(style).read()
            frameless = self.getPref("UI","Frameless",False)
            if frameless:
                ss = "%s%s" % (ss,"""
QMenuBar { padding-left: 65px; }
""")
            else:
                ss = "%s%s" % (ss,"""
QMenuBar { border-radius: 0px; }
QStatusBar { border-radius: 0px; }
QStatusBar QSizeGrip { border-bottom-right-radius: 0px; }
""")
            self.app.setStyleSheet(ss)
        except Exception,e:
            pass
    def exec_(self):
        self.initialize()
        self.ui.dockWidget.setWindowTitle("R-Project")
        self.myMainWindow.show()
        self.SetStatusText("OK")
        self.app.exec_()
    def initialize(self):
        self.RestoreUIFromConfig()
    def finalize(self):
        # TODO
        # Save GUI
        self.SaveUI2Config()
        # Save Config
        self.SaveConfig()
    def addComponent(self,c):
        tmp_page = c.getPanel()
        self.components[c.getName()]=[c,tmp_page]
        
        QtGui.QListWidgetItem( c.getIcon(), c.getName(), self.ui.listWidget )
        tmp_page = self.components[c.getName()][1]
        self.ui.stackedWidget.addWidget(tmp_page)
        if self.ui.dockWidget.isHidden() and len(self.components)>1:
            self.ui.dockWidget.show()
        elif not self.ui.dockWidget.isHidden() and len(self.components)==1:
            self.ui.dockWidget.hide()
        menu = c.getMenu(self.ui.menubar)
    def SetupPlugins(self, plugins_path):
        self._checkPath(plugins_path)
        if not plugins_path in sys.path:
            sys.path.append(plugins_path)
# ############################## ... ###################################
    def Alert(self,msg, eccezione=None):
        if self.app.thread()==QtCore.QThread.currentThread():
            if not eccezione is None:
                QtGui.QMessageBox.warning(\
                    self.myMainWindow, "Attenzione",\
                    "%s\n\nEccezione: %s\n\n%s" % (msg, eccezione,"".join(traceback.format_tb(sys.exc_info()[2])))\
                )
            else:
                QtGui.QMessageBox.information(self.myMainWindow, "Informazione", msg)
        else:
            self.emit(QtCore.SIGNAL('alert(PyQt_PyObject,PyQt_PyObject)'),msg,eccezione)
    def Confirm(self,msg):
        ret = QtGui.QMessageBox.question(self.myMainWindow, "Domanda", msg, QtGui.QMessageBox.Cancel|QtGui.QMessageBox.Ok)
        return ret==QtGui.QMessageBox.Ok
    def SetStatusText(self,msg):
        self.ui.statusbar.showMessage(msg)
    def SetProgress(self,perc):
        if perc <= 0:
            if self.progressBar.maximum()!=0:
                self.progressBar.setMaximum(0)
        else:
            if self.progressBar.maximum()!=100:
                self.progressBar.setMaximum(100)
            self.progressBar.setValue(perc)
# ############################## PREFERENCES ###################################
    def SaveUI2Config(self):
        for l in self.components.values():
            l[0].SaveUI2Config()
        # Main Window
        # TODO self.setPref("UI","searchFolder",self.ui.searchFolder.text())
        # TODO self.setPref("UI","comboForms.currentIndex",self.ui.comboForms.currentIndex())
        self.setPref("UI","maximized",self.myMainWindow.isMaximized())
        if not self.myMainWindow.isMaximized():
            self.setPref("UI","mainWindowSize",[self.myMainWindow.width(),self.myMainWindow.height()])
        # Login Dialog
        self.setPref("UILogin","login",self.uiLogin.lineEdit.text())
        self.setPref("UILogin","pwd",self.uiLogin.lineEdit_2.text())
        self.setPref("DB","schema",self.uiLogin.lineEdit_schema.text())
        self.setPref("UILogin","comboBox.currentIndex",self.uiLogin.comboBox.currentIndex())
        self.setPref("UILogin","comboBox.currentText",self.uiLogin.comboBox.currentText())
    def RestoreUIFromConfig(self):
        for l in self.components.values():
            l[0].RestoreUIFromConfig()
        # Main Window
        # TODO self.ui.searchFolder.setText(self.getPref("UI","searchFolder",""))
        # TODO self.ui.comboForms.setCurrentIndex( self.getPref("UI","comboForms.currentIndex","0"))
        if self.getPref("UI","maximized",False):
            self.myMainWindow.showMaximized()
        # Login Dialog
        self.uiLogin.lineEdit.setText(self.getPref("UILogin","login",""))
        self.uiLogin.lineEdit_2.setText(self.getPref("UILogin","pwd",""))
        self.uiLogin.lineEdit_schema.setText( self.getPref('DB','schema','rprj') )
        serverlist = self.getPref("UILogin","servers",["sqlite:%s"%RPRJ_LOCAL_DB,"http://localhost/rprj/","mysql:localhost:rproject:root:","postgresql:localhost:rproject:roberto:"])
        for s in serverlist:
            self.uiLogin.comboBox.addItem(s)
        self.uiLogin.comboBox.setCurrentIndex( self.getPref("UILogin","comboBox.currentIndex","0"))
        self.uiLogin.comboBox.setEditText(self.getPref("UILogin","comboBox.currentText",serverlist[0]))
    def getPref(self,section,option,default=None):
        if isinstance(default,bool): default="%s" % (default)
        if not self.config.has_section(section):
            self.config.add_section(section)
        if not self.config.has_option(section,option):
            self.config.set(section,option,default)
        tmp = self.config.get(section,option,default)
        ret = tmp
        try:
            pycode = "ret = %s" % (tmp)
            exec(compile(pycode, "myeval.py","exec"))
        except Exception,e:
            pass
        return ret
    def setPref(self,section,option,valore):
        if not self.config.has_section(section):
            self.config.add_section(section)
        if isinstance( valore, bool ): valore ="%s"%(valore)
        self.config.set(section,option,valore)
    def _checkPath(self,mycfgfile):
        dirs = mycfgfile.split(os.path.sep)[:-1]
        tmpdir = []
        for d in dirs:
            if d == "":
                continue
            tmpdir.append(d)
            tmpstr = "%s%s" % (os.path.sep, os.path.sep.join(tmpdir))
            if sys.platform=='win32':
                tmpstr = os.path.sep.join(tmpdir)
            if not os.path.exists(tmpstr):
                os.mkdir(tmpstr)
        return os.path.exists(tmpstr)
    def LoadConfig(self,config_file):
        self.config_filename = config_file
        self._checkPath(self.config_filename)
        self.config = ConfigParser()
        try:
            self.config.readfp(file(self.config_filename,'rt'))
        except Exception,e:
            print e
    def SaveConfig(self):
        self._checkPath(self.config_filename)
        self.config.write( file(self.config_filename,'w') )
# ############################## LOGIC ###################################
    def connectToServer(self,url,user,pwd):
        myurl = url
        if myurl.startswith("http"):
            if myurl.endswith("/xmlrpc_server.php"):
                pass # OK
            elif myurl.endswith("/"):
                myurl = "%sxmlrpc_server.php" % ( myurl )
            else:
                myurl = "%s/xmlrpc_server.php" % ( myurl )
        # Connection
        myconn = createConnection(myurl,self.getPref("Debug","connection",False)) #True)
        # Server
        self.server = dbschema.ObjectMgr(myconn, self.getPref("Debug","server",False), self.getPref('DB','schema','rprj'))
        self.server.setDBEFactory(self.dbeFactory)
        self.server.connect()
        myerr = ""
        #print self.server.ping()
        try:
            self.server.login(user,pwd)
        except Exception,e:
            myerr = "%s" % e
            print myerr
            print "".join(traceback.format_tb(sys.exc_info()[2]))
        if not self.server.user is None:
            # New server?
            serverlist = self.getPref("UILogin","servers",["http://localhost/rprj/","sqlite:%s"%RPRJ_LOCAL_DB,"mysql:localhost:rproject:root:","postgresql:localhost:rproject:roberto:"])
            if not url in serverlist:
                serverlist.insert(0,url)
                self.setPref("UILogin","servers",serverlist)
            # Form Factory
            formFactory = FormFactory()
            for f in formschema.formschema_type_list.keys():
                formFactory.register(f,formschema.formschema_type_list[f], self.server)
            for k in self.components.keys():
                mycomponent,  mypanel = self.components[k]
                mycomponent.connectToServer(self.server, formFactory)
            return "Logged in: %s (%s)" % (self.server.user.getValue('login'), self.server.user.getValue('fullname'))
        elif len(myerr)>0:
            return "Login error: %s."%myerr
        else:
            return "Login error!"

# ############################## SLOTS ###################################
    def slotLoginOk(self):
        url = "%s" % self.uiLogin.comboBox.currentText()
        user = "%s" % self.uiLogin.lineEdit.text()
        pwd = "%s" % self.uiLogin.lineEdit_2.text()
        schema = "%s" % self.uiLogin.lineEdit_schema.text()
        self.setPref("DB","schema",schema)
        msg = self.connectToServer(url,user,pwd)
        self.SetStatusText(msg)
    def slotLoginCancel(self):
        pass
    def slotLogin(self):
        if not self.loginDialog.show():
            self.loginDialog.show()
        else:
            self.loginDialog.hide()
    def slotLoginMoreOptions(self):
        self._loginShowOptions( self.uiLogin.pushButton.isChecked() )

    def slotQuit(self):
        if not self.Confirm("Confirm Quit?"):
            return
        self.myMainWindow.close()
        self.finalize()
    def slotMinimizeWindow(self):
        self.myMainWindow.showMinimized()
    def slotMaximizeWindow(self):
        if self.myMainWindow.isMaximized():
            self.myMainWindow.showNormal()
        else:
            self.myMainWindow.showMaximized()

    def slotReloadStylesheet(self):
        self.loadStyle()




if __name__=='__main__':
    if len(sys.argv)>1:
        RPRJ_CONFIG = sys.argv[1]
    myapp = RApp()
    myapp.exec_()
